
/**
 * Extract all the mentions from markdown text.
 * Mentions are of the form [{ _id, collection }].
 *
 * TODO:
 *   - support post mentions
 */

var mdb = require('../databases/mongodb')

// get all the mentions from the markdown
module.exports = function* (markdown) {
  if (!markdown || typeof markdown !== 'string') return []

  var mentions = []

  // search pages collection
  var pages$or = (markdown.match(/(?:^|\s)(?:#|@)(\w{3,30}|@\w{1,15})\b/g) || [])
    .map(pageSlugToQuery)
  if (pages$or.length) {
    mentions.push(mdb.collection('pages').find({
      deleted: false,
      $or: pages$or,
    }).fields({
      _id: true,
      collection: true
    }))
  }

  mentions = yield mentions
  return mentions.reduce(concat, [])
}

function pageSlugToQuery(slug) {
  slug = slug.toLowerCase()
  if (slug[0] === '@') {
    return {
      'twitter.screen_name': slug.slice(1)
    }
  } else {
    return {
      slug: slug
    }
  }
}

function concat(a, b) {
  return a.concat(b)
}
