
/**
 * Password reset codes.
 */

var crypto = require('mz/crypto')
var pswd = require('pswd')

var mdb = require('../databases/mongodb')

// reset codes last an hour
var RESET_CODE_MAXAGE = 60 * 60 * 1000

exports.create = function () {
  return crypto.pseudoRandomBytes(16).then(toHex)
}

function toHex(buf) {
  return buf.toString('hex')
}

exports.update = function* (user, code) {
  var hash = yield pswd.hash(code)
  return yield mdb.findOne('_id', user._id)
  .set('reset', {
    code: hash,
    expire: new Date(Date.now() + RESET_CODE_MAXAGE)
  })
}

exports.check = function* (user, code) {
  var reset = user.reset
  if (!reset) return false
  if (Date.now() > reset.expire.getTime()) return false
  return yield pswd.compare(code, reset.code)
}
