
/**
 * Get all positive and negative votes for a thing and type.
 * Checks a user's followers.
 */


var ObjectID = require('mongodb').ObjectID

var redis = require('../databases/redis')

module.exports = function* (user, thing, type) {
  var user_id = user._id.toHexString()
  var thing_id = thing._id.toHexString()
  var following_key = 'bookmark:following:' + user_id
  var votee_pos_key = 'votee:pos:' + type + ':' + thing_id
  var votee_neg_key = 'votee:neg:' + type + ':' + thing_id

  // output sets
  var out = {
    pos: 'friend:votes:' + thing_id + ':' + type + ':' + user_id + ':pos',
    neg: 'friend:votes:' + thing_id + ':' + type + ':' + user_id + ':neg',
  }

  // union
  var count = yield {
    pos: redis.zunionstore(out.pos, 2, following_key, votee_pos_key),
    neg: redis.zunionstore(out.pos, 2, following_key, votee_neg_key),
  }

  // get the results
  var ids = yield {
    pos: count.pos ? redis.zrevrange(out.pos, 0, -1) : [],
    neg: count.neg ? redis.zrevrange(out.neg, 0, -1) : []
  }

  // delete the temporary set
  yield [
    redis.del(out.pos),
    redis.del(out.neg)
  ]

  // output
  ids.pos = ids.pos.map(toOID)
  ids.neg = ids.neg.map(toOID)
  return ids
}

function toOID(x) {
  return new ObjectID(x)
}
